#include <stdio.h>

#include "mpi.h"
#include "mpptest.h"

#if HAVE_STDLIB_H
#include <stdlib.h>
#endif

/* 
   Test of single process memcpy.
   ctx is ignored for this test.

   Note that the test is run *once* before it is timed.  This
   is very important to ensure that all of the data is paged in and 
   ready to go.  Without this, a large number of repititions are needed
   to ensure that any "cold-start" cost is averaged out (almost) across 
   many operations (i.e., you need a large value of reps otherwise).
*/
double memcpy_rate(int reps, int len, void *ctx)
{
  double elapsed_time;
  int  i;
  char *sbuffer,*rbuffer;
  double t0;

  sbuffer = (char *)malloc(len+4);
  rbuffer = (char *)malloc(len+4);

  memcpy( rbuffer, sbuffer, len );
  t0 = MPI_Wtime();
  for(i=0;i<reps;i++){
      memcpy( rbuffer, sbuffer, len );
  }
  elapsed_time = MPI_Wtime() - t0;

  free(sbuffer);
  free(rbuffer);
  return(elapsed_time);
}

double memcpy_rate_int(int reps, int len, void *ctx)
{
  double elapsed_time;
  int  k, i, ilen;
  int *sbuffer,*rbuffer;
  double t0;

  ilen = 1 + len / sizeof(int);
  sbuffer = (int *)malloc(ilen*sizeof(int));
  rbuffer = (int *)malloc(ilen*sizeof(int));

  for (k=0; k<ilen-1; k++)
      rbuffer[k] = sbuffer[k];
  t0 = MPI_Wtime();
  for(i=0;i<reps;i++){
    for (k=0; k<ilen-1; k++)
      rbuffer[k] = sbuffer[k];
  }
  elapsed_time = MPI_Wtime() - t0;

  free(sbuffer);
  free(rbuffer);
  return(elapsed_time);
}

double memcpy_rate_double(int reps, int len, void *ctx)
{
  double elapsed_time;
  int  k, i, ilen;
  double *sbuffer,*rbuffer;
  double t0;

  ilen = 1 + len / sizeof(double);
  sbuffer = (double *)malloc(ilen*sizeof(double));
  rbuffer = (double *)malloc(ilen*sizeof(double));

  for (k=0; k<ilen-1; k++)
      rbuffer[k] = sbuffer[k];
  t0 = MPI_Wtime();
  for(i=0;i<reps;i++){
    for (k=0; k<ilen-1; k++)
      rbuffer[k] = sbuffer[k];
  }
  elapsed_time = MPI_Wtime() - t0;

  free(sbuffer);
  free(rbuffer);
  return(elapsed_time);
}
